# 搭建基本 Spring RestAPI 项目

## 运行效果

~~~ bash
# 测试基本 HTTP GET
curl -XGET localhost:8080/hi
# 结果 {"msg":"hi"}


# 测试子路由
curl -XGET localhost:8080/sub/hi
# 结果 {"msg":"hi, sub route"}


# 测试 URL Query
curl -XGET localhost:8080/hello?name=luo
# 返回 {"msg":"hi, luo"}


# 测试 JSON 内容
curl -XPOST \
-H 'Content-Type: application/json' \
-d '{"name": "luo"}' \
localhost:8080/hello
# 结果 {"msg": "welcome back, luo"}


# 测试占位符变量
curl -XGET localhost:8080/profile/luo
# 结果 {"msg": "profile of user: luo"}
~~~


## 步骤

### Gradle 新建 Java Application 项目

~~~ bash
gradle init

# 问项目类型: 选择 2(application)
# 问开发语言: 选择 3(Java)
# 问有无子项目: 选择 1(no)
# 问构建语言: 选择 1(Groovy)
# 问测试框架: 默认
# 问项目名称: 默认
# 问包名: org.luo3.springrestdemo
~~~

### [`app/build.gradle`](./app/build.gradle) 增加 SpringBoot 插件

~~~ gradle
plugins {
  // ...

  // [添加] SpringBoot Gradle 插件
  id "org.springframework.boot" version "2.4.8"
}
~~~

### [`app/build.gradle`](./app/build.gradle) 增加 SpringBoot starter web 依赖

~~~ gradle
dependencies {
    // [增加] SpringBoot Starter Web 模块整合包
    implementation 'org.springframework.boot:spring-boot-starter-web:2.5.5'

    // ...
}
~~~

### [`App.java`](./app/src/main/java/org/luo3/springrestdemo/App.java) 入口类启动 SpringBootApplication

~~~ java
@SpringBootApplication
public class App {
  public static void main(String[] args) {
    SpringApplication.run(App.class, args);
  }
}
~~~

### 测试启动，检查 SpringBoot 是否正常运行

~~~ bash
./gradlew bootRun
~~~

CTRL+C 关闭程序，继续开发控制器。

### 新建 [`AppController.java`](./app/src/main/java/org/luo3/springrestdemo/AppController.java)

~~~ java
// @RestController 标记为 RestAPI 控制器
@RestController
public class AppController {

  // 基本 HTTP GET
  @GetMapping("/hi")
  public Map<String, String> handleBaseHTTPGET () {
    return new HashMap<>(){{
      put("msg", "hi");
    }};
  }

  // URL Query
  @GetMapping("/sub/hello")
  public Map<String, String> handleURLQuery (@RequestParam("name") String name) {
    return new HashMap<>(){{
      put("msg", "hi, " + name);
    }};
  }

  // JSON body
  @PostMapping(
    path = "/hello",
    consumes = "application/json", 
    produces = "application/json"
  )
  public Map<String,Object> handleJSONBody (@RequestBody Map<String,Object> jsonObject) {
    String name = jsonObject.get("name").toString();
    return new HashMap<>(){{
      put("msg", "welcome back, " + name);
    }};
  }

  // 占位符
  @GetMapping("/var/{slug}")
  public Map<String, String> handlePathVariable (@PathVariable("slug") String slugValue) {
    return new HashMap<>(){{
      put("msg", "profile of user: " + slugValue);
    }};
  }

}
~~~

### 子路由控制器 [`SubRouteController.java`](./app/src/main/java/org/luo3/springrestdemo/SubRouteController.java)

~~~ java
@RestController
public class SubRouteController {
  
  // 子路由
  @GetMapping("/sub/hi")
  public Map<String, String> handleSubRoute () {
    return new HashMap<>(){{
      put("msg", "hi, sub route");
    }};
  }

}
~~~

### 启动应用，测试请求结果

~~~ bash
./gradlew bootRun
~~~

### 打包成 Jar，可独立运行

~~~ bash
./gradlew bootJar
~~~
